package com.vieaura.plugins.rw_barcode_scanner

import android.util.Log
import android.app.Activity
import android.content.Intent
import android.os.Bundle
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar


class BarcodeScannerPluginActivity : Activity() {
    private val BARCODE_REQUEST = 10011
    private val ACTION_BARCODE = "com.realwear.barcodereader.intent.action.SCAN_BARCODE"
    private val EXTRA_RESULT = "com.realwear.barcodereader.intent.extra.RESULT"

    //
    // Available barcode symbologies
    //
    private val EXTRA_CODE_128 = "com.realwear.barcodereader.intent.extra.CODE_128"
    private val EXTRA_CODE_DM = "com.realwear.barcodereader.intent.extra.CODE_DM"
    private val EXTRA_CODE_EAN_UPC = "com.realwear.barcodereader.intent.extra.CODE_EAN_UPC"

    private val EXTRA_CODE_QR = "com.realwear.barcodereader.intent.extra.CODE_QR"

    private var scanType = "default"

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        if (intent.hasExtra("scanType")) {
            scanType = intent.getStringExtra("scanType")!!
        }

        Log.v("BarcodeScannerPluginActivity", "scanType=" + scanType);
        // Set which symbologies are enabled. If none is specified, all are enabled by default
        //if QR 

        val intentBarcode = Intent(ACTION_BARCODE)
        if (scanType == "qr") {
            intentBarcode.putExtra(EXTRA_CODE_QR, true)

        } else if (scanType == "upc") {
            // UPC 
            intentBarcode.putExtra(EXTRA_CODE_EAN_UPC, true)

        } else if (scanType == "barcode") {
            // 128 or DM
            intentBarcode.putExtra(EXTRA_CODE_128, true)
            intentBarcode.putExtra(EXTRA_CODE_DM, true)

        } else {
            intentBarcode.putExtra(EXTRA_CODE_QR, true)
            intentBarcode.putExtra(EXTRA_CODE_128, true)
            intentBarcode.putExtra(EXTRA_CODE_DM, true)
            intentBarcode.putExtra(EXTRA_CODE_EAN_UPC, true)
        }

        startActivityForResult(intentBarcode, BARCODE_REQUEST)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, intent: Intent?) {
        var result: String? = "[No Barcode]"
        if (resultCode == RESULT_OK && requestCode == BARCODE_REQUEST) {
            if (intent != null) {
                result = intent.getStringExtra(EXTRA_RESULT)
            }
        }

        RwBarcodeScannerPlugin.onBarcodeResult(result)
        finish()
    }

}

class RwBarcodeScannerPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    //lateinit var activity: Activity?
    private var activity: Activity? = null

    companion object {
        private const val CHANNEL = "com.vieaura.plugins/rw_barcode_scanner"
        private lateinit var methodResult: Result

        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), CHANNEL)
            val plugin = RwBarcodeScannerPlugin()

            plugin.activity = registrar.activity()
            channel.setMethodCallHandler(plugin)
        }

        @JvmStatic
        fun onBarcodeResult(content: String?) {
            methodResult.success(content)
        }
    }

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        val channel =
            MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), CHANNEL)

        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "scan" -> {
                val text = call.argument<String>("scanType")
                Log.i("RwBarcodeScannerPlugin", "scanType: " + text); // qr/upc/barcode

                if (activity != null) {
                    val intent = Intent(activity, BarcodeScannerPluginActivity::class.java)
                    if (text != null) intent.putExtra("scanType", text)
                    methodResult = result
                    activity?.startActivity(intent)
                }
            }
            else ->
                result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        this.activity = binding.activity
    }

    override fun onDetachedFromActivity() {
    }

    override fun onDetachedFromActivityForConfigChanges() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }
}
